# _AC_FC_CHECK_SIZEOF (TYPE, [IGNORED], [INCLUDES = DEFAULT-INCLUDES])
# --------------------------------------------------------------------
#
# Determine the size of types for the Fortran.
# First the binary representation of TYPE is written to conftest.unf between
# two a. After that conftest.unf is reread and the distance between 2 'a' gives
# the type length. In the end the detected size is put in conftest.res.
#
# It will fail if there is a 'a' in binary format in the header of 
# conftest.unf, if there is a 'a' or a newline in the binary representation.

AC_DEFUN([_AC_FC_BIGENDIAN],
[_AC_FORTRAN_ASSERT()dnl
AC_CACHE_CHECK([whether byte ordering is bigendian],
               [ac_cv_[]_AC_LANG_ABBREV[]_bigendian],
[AC_RUN_IFELSE([AC_LANG_SOURCE([[      program endian

      integer stat
      integer asciiabcd
      integer ebcdicabcd
      character abcd*4
      character dcba*4
      character string*30
      integer bigendian,litendian
      integer sbigindex,slitindex
      logical fclosed
      integer res

      abcd = 'abcd'
      dcba = 'dcba'
      asciiabcd=1684234849
      ebcdicabcd=-2071756159
      bigendian = 0
      litendian = 0
      sbigindex = 0
      slitindex = 0
      fclosed = .False.

      open (10,FORM='UNFORMATTED',FILE='conftest.unf',STATUS='NEW',
     +IOSTAT=stat,ERR=100)
      if (stat .ne. 0) goto 100
      write(10) asciiabcd, ebcdicabcd
      close(10,IOSTAT=stat,ERR=100)
      if (stat .ne. 0) goto 100

1000  format (A)
      open (10,FILE='conftest.unf',STATUS='OLD',IOSTAT=stat,ERR=100)
      if (stat .ne. 0) goto 100

      do 20 k=1,6
        read (10, 1000, END=120) string

        goto 130

120     close(10,IOSTAT=stat,ERR=100)
        if (stat .ne. 0) goto 100
        fclosed = .True.

130     continue
        do 10 j=1,len(string)
        if (sbigindex .gt. 0) then
          if (string(j:j) .ne. dcba(1+sbigindex:1+sbigindex)) then
             bigendian = 2
             sbigindex = 0
          else
             sbigindex = sbigindex + 1
             if (sbigindex .eq. len(dcba)) then
               bigendian = 1
               sbigindex = 0
             endif
          endif
        elseif (slitindex .gt. 0) then
          if (string(j:j) .ne. abcd(1+slitindex:1+slitindex)) then
             litendian = 2
             slitindex = 0
          else 
             slitindex = slitindex + 1
             if (slitindex .eq. len(abcd)) then
               litendian = 1
               slitindex = 0
             endif
          endif
        elseif (string(j:j) .eq. dcba(1:1)) then
          if (bigendian .eq. 0) then
             sbigindex = 1
          endif
        elseif (string(j:j) .eq. abcd(1:1)) then
          if (litendian .eq. 0) then
             slitindex = 1
          endif
        endif
10      continue
      if (fclosed) goto 140
20    continue

140   res = 0
      if (litendian .eq. 1 .and. bigendian .eq. 1) then
         res = 3
      elseif (bigendian .eq. 1) then
         res = 1
      elseif (litendian .eq. 1) then
         res = 2
      endif
     
1100  format (I4)
      open (10,FILE='conftest.res',STATUS='NEW',IOSTAT=stat,ERR=100)
      if (stat .ne. 0) goto 100
      write (10, 1100) res
      close(10)

100   continue
      stop
      end]]
)],
[ if test -f 'conftest.res'; then
   AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian_result)=`cat conftest.res | sed 's/ *//'`
   rm -f conftest.res
   if test z$AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian_result) = 'z1'; then
     AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian)='yes'
   elif test z$AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian_result) = 'z2'; then
     AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian)='no'
   else
     AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian)='unknown'
  fi
  else 
     AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian)='unknown'
  fi
],
[
  AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian)='unknown'
], [])
rm -f conftest.unf])

case $AS_TR_SH(ac_cv_[]_AC_LANG_ABBREV[]_bigendian) in
  yes)
    m4_default([$1],
      [AC_DEFINE([WORDS_BIGENDIAN], 1,
        [Define to 1 if your processor stores words with the most significant
         byte first (like Motorola and SPARC, unlike Intel and VAX).])]) ;;
  no)
    $2 ;;
  *)
    m4_default([$3],
      [AC_MSG_ERROR([unknown endianness
presetting ac_cv_[]_AC_LANG_ABBREV[]_bigendian=no (or yes) will help])]) ;;
esac
])# _AC_FC_BIGENDIAN 


# AC_F77_BIGENDIAN
# -------------------
AC_DEFUN([AC_F77_BIGENDIAN],
[AC_REQUIRE([AC_PROG_F77])dnl
AC_LANG_PUSH(Fortran 77)dnl
_AC_FC_BIGENDIAN([$1],[$2],[$3])
AC_LANG_POP(Fortran 77)dnl
])# AC_F77_CHECK_SIZEOF


# AC_FC_BIGENDIAN
# ------------------
AC_DEFUN([AC_FC_BIGENDIAN],
[AC_REQUIRE([AC_PROG_FC])dnl
AC_LANG_PUSH(Fortran)dnl
_AC_FC_BIGENDIAN([$1],[$2],[$3])
AC_LANG_POP(Fortran)dnl
])# AC_FC_BIGENDIAN


